from distutils.core import setup

setup(
    name='ftwgl',
    packages=['ftwgl'],
    version='0.1.11',
    license='MIT',
    description='FTW Python Client',
    author='solitary',
    author_email='solitaryurt@gmail.com',
    url='https://ftwgl.net',
    keywords=['Gaming League', 'FTWGL'],
    install_requires=[
        'aiohttp'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
)
